/*
 * Copyright (C) 2006 - Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <qdialog.h>
#include <qfiledialog.h>
#include <qstring.h>
#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <trim.h>
#include <qmessagebox.h>
#include <qinputdialog.h>
using namespace std;

void janela_principal::abreArquivo()
{    
    ifstream arquivo;
    std::string i;
    
    // Exibe o dialogo para o usuario escolher o arquivo (apenas .obj)
    QFileDialog *dlg = new QFileDialog(this);
        dlg->setCaption( QFileDialog::tr( "Open" ) );
        dlg->setMode( QFileDialog::ExistingFile );
	dlg->setFilter( "All C++ files (*.obj)" );
	
        QString pathArquivo;
        if ( dlg->exec() == QDialog::Accepted ) {
            pathArquivo = dlg->selectedFile();
        }
        delete dlg;	
	
	// Exibe o caminho do arquivo na janela	
	labelArquivo->setText(pathArquivo);
	
	// Abre o arquivo escolhido
	arquivo.open(pathArquivo);
	
	if(!arquivo) {
		cerr << "Falha ao abrir o arquivo";
		return;
	}
	
	// Remove os dados da tabela de codigos (programa
	for(int k = tabelaCodigos->numRows(); k >= 0;  k--)
	    tabelaCodigos->removeRow(k);
	
	// Limpa as entradas e saidas de dados
	entradaDados->clear();
	saidaDados->clear();
	
	// Remove as entradas da tabela de memoria
	for(int k=tabelaMemoria->numRows(); k >= 0; k--)
	    tabelaMemoria->removeRow(k);
    
	// Pega linha por linha do arquivo e insere na tabela de codigos
	for(int j=0; getline(arquivo, i, '\n'); j++) {
	    tabelaCodigos->insertRows(j, 1);
	    std::string p = i.substr(0, 4);
	    tabelaCodigos->setText(j, 0, trim(p));
	    p = i.substr(4, 8);	    	    
	    tabelaCodigos->setText(j, 1, trim(p));
    	    p = i.substr(12, 4);	    
    	    tabelaCodigos->setText(j, 2, trim(p));
    	    p = i.substr(16, 4);
        	    tabelaCodigos->setText(j, 3, trim(p));
	}
	botao_iniciar->setEnabled(true);
	arquivo.close();
}

void janela_principal::iniciar()
{
    // Indice da regiao de prorgama
    int i=0;
    
    // Indice da pilha de dados
    int s=0;
    
    // Variaveis uteis
    int valor1=0,  valor2=0;
    int k = 0, n = 0, m =0, t = 0;
    bool ok;
    
    // Instrucao atual
    QString instrucao;
    
    // Opcoes padroes de inicializacao
    botao_carregar->setEnabled(false);
    botao_iniciar->setEnabled(false);
    botao_iniciar->setEnabled(false);
    botao_parar->setEnabled(true);
    modoExecucao->setEnabled(false);

    // Limpa as entradas e saidas de dados
    entradaDados->clear();
    saidaDados->clear();

    // Remove as entradas da tabela de memoria
    for(int k=tabelaMemoria->numRows(); k >= 0; k--)
        tabelaMemoria->removeRow(k);
	

    // Percorre as linhas da tabela de codigos (programa)
    for(i=0; i < tabelaCodigos->numRows(); i++) {
	instrucao = tabelaCodigos->text(i, 1);
	
	// Seleciona a linha que esta sendo executada no programa
	tabelaCodigos->clearSelection(true);
	tabelaCodigos->selectRow(i);
	tabelaCodigos->setCurrentCell(i, 0);
		
	// Atualiza os indices de programa e memoria	
	labelPrograma->setText("Codigo de Maquina: i = " + QString::number(i));
	labelMemoria->setText("Memoria: s = " + QString::number(s));
	
	// Verifica qual modo de execucao (0 = normal; 1 = passo a passo)
	if(modoExecucao->currentItem() != 0) {
	    if(QMessageBox::information( this, "Modo de execucao",
                                      "Continuar executando ?",
                                      "Nao", "Sim",  0, 1 ) != 1)
		break;
	}
	
	// Execucao das instrucoes
	if(instrucao == "ALLOC") {
    	    m = tabelaCodigos->text(i, 2).toInt(&ok, 10);
	    n = tabelaCodigos->text(i, 3).toInt(&ok, 10);
	    for(k=0; k <= n-1; k++) {
		s = s+1;
		tabelaMemoria->insertRows(tabelaMemoria->numRows(), 1);
		valor1 = tabelaMemoria->text(m+k, 0).toInt(&ok, 10);
		tabelaMemoria->setText(s, 0, QString::number(valor1));
	    }
	    continue;
	}
	if(instrucao == "DALLOC") {
    	    m = tabelaCodigos->text(i, 2).toInt(&ok, 10);
	    n = tabelaCodigos->text(i, 3).toInt(&ok, 10);
	    for(k=n-1; k >= 0; k--) {
		valor1 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
		tabelaMemoria->setText(m+k, 0, QString::number(valor1));
		s = s-1;
	    }
	    continue;
	}
	if(instrucao == "STR") {
    	    n = tabelaCodigos->text(i, 2).toInt(&ok, 10);
	    valor1 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    tabelaMemoria->setText(n, 0, QString::number(valor1));
	    s = s -1;
	    continue;
	}   
	if(instrucao == "RETURN") {
	    i = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    s = s - 1;
	    continue;
	}
	if(instrucao == "CALL") {
    	    t = getLabelLine(tabelaCodigos->text(i, 2));
	    s = s + 1;
	    tabelaMemoria->insertRows(tabelaMemoria->numRows(), 1);
	    tabelaMemoria->setText(s, 0, QString::number(i));
	    i = t;
	    continue;
	}
	if(instrucao == "JMP") {
	     t = getLabelLine(tabelaCodigos->text(i, 2));
	     i = t;
	     i--;
	     continue;
	 }
	if(instrucao == "JMPF") {
	    valor1 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 == 0) {
		t = getLabelLine(tabelaCodigos->text(i, 2));
		i = t;
		i--;
	    }//else
	//	i = i + 1;
	    s = s - 1;
	    continue;
	 }
	if(instrucao == "NULL") {
	    continue;
	}
	if(instrucao == "RD") {
	    s = s + 1;
	    tabelaMemoria->insertRows(tabelaMemoria->numRows(), 1);
	    t = QInputDialog::getInteger("Ler entrada", "Entrada", 0, -2147483646, 2147483646, 1, &ok, this );
	    tabelaMemoria->setText(s, 0, QString::number(t));
	    entradaDados->insertItem(QString::number(t), entradaDados->numRows()+1);
	    if(!ok)
		break;
	    continue;
	}
	if(instrucao == "PRN") {
	    valor1 = tabelaMemoria->text(s, 0).toInt(&ok, 10);	    
	    s = s - 1;	    
	    saidaDados->insertItem(QString::number(valor1), saidaDados->numRows()+1);
	    continue;
	}	    
	if(instrucao == "LDC") {
	    s = s+1;
	    tabelaMemoria->insertRows(tabelaMemoria->numRows(), 1);
    	    k = tabelaCodigos->text(i, 2).toInt(&ok, 10);
    	    tabelaMemoria->setText(s, 0, QString::number(k));
	}
	if(instrucao == "LDV") {
	   s = s+1;
	   tabelaMemoria->insertRows(tabelaMemoria->numRows(), 1);
	   n = tabelaCodigos->text(i, 2).toInt(&ok, 10);
	   valor1 = tabelaMemoria->text(n, 0).toInt(&ok, 10);
    	   tabelaMemoria->setText(s, 0, QString::number(valor1));
	}
	if(instrucao ==  "ADD") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    valor1 = valor1 + valor2;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));	    
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "SUB") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    valor1 = valor1 - valor2;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "MULT") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    valor1 = valor1 * valor2;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "DIVI") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    valor1 = valor1 / valor2;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "INV") {
	    valor1 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    valor1 = valor1 * -1;
	    tabelaMemoria->setText(s, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "AND") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 == 1 && valor2 == 1)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "OR") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 == 1 || valor2 == 1)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "NEG") {
	    valor1 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    valor1 = 1- valor1;
	    tabelaMemoria->setText(s, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "CME") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 < valor2)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "CMA") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 > valor2)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "CEQ") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 == valor2)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "CDIF") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 != valor2)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "CMEQ") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 <= valor2)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao ==  "CMAQ") {
	    valor1 = tabelaMemoria->text(s-1, 0).toInt(&ok, 10);
	    valor2 = tabelaMemoria->text(s, 0).toInt(&ok, 10);
	    if(valor1 >= valor2)
		valor1 = 1;
	    else
		valor1 = 0;
	    tabelaMemoria->setText(s-1, 0, QString::number(valor1));
	    s=s-1;
	    continue;
	}
	if(instrucao == "START") {
	    s = -1;
	    continue;
	}
	if(instrucao == "HLT") {
	    break;
	}
    }
    
    botao_carregar->setEnabled(true);
    botao_iniciar->setEnabled(true);
    botao_parar->setEnabled(false);   
    modoExecucao->setEnabled(true);
}

/* Percorre a tabela de codigos (programa) e tenta encontrar o label, se 
** encontrar, retorna a linha que tem o label
** Entrada: QString label
** Saida: int com o numero da linha aonde esta o label
*/
int janela_principal::getLabelLine( QString label)
{
    QString l;
    int i;
    
    for(i=0; i < tabelaCodigos->numRows(); i++) {
	l = tabelaCodigos->text(i, 0);
	if( l == label )
	    return i;
    }
    return -1;
}

/* 
** Janela sobre
*/
void janela_principal::sobre()
{
    QString msg = "<b><font size='+2'>Sobre</font>\
		<br><font size='+1'>Maquina Virtual - Compiladores 2006</font></b>\
		<br>Pontificia Universidade Catolica de Campinas\
		<br>Professor: Ricardo Freitas\
		<br><br><b>Autores:</b>\
		<br>Bruno Mazzoco - RA: 02150522 - brunomazzoco@gmail.com\
		<br>Cesar Kallas - cesarkallas@soulivre.org - RA:02099224\
		<br>Renato Demarco - RA: 02142818 - renatodemarco@gmail.com\
		<br><br>This program is free software; you can redistribute\
		<br>it and or modify it under the terms of the GNU General\
		<br>Public License as published by the Free Software Foundation;\
		<br>either version 2 of the License, or (at your option) any later version.\
		<br>";
    QMessageBox::information( this, "MquinaVirtual - Sobre",  msg, "&Fechar", 0, 0, 0, 1);
}
